/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004-2005  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Id: tverifyhash.c,v 1.1 2005/04/25 12:03:44 kasperd Exp $
*/

#include "tstore.h"

int main(int argc, char ** argv)
{
  struct store *s;
  int hash_fd;
  uint8_t buf[HBS];
  uint64_t i;

  if (argc != 2) {
    fprintf(stderr,"Uage: %s <store>\n",argv[0]);
    return 1;
  }

  s=open_store(argv[1],O_RDONLY,CHECK_LOCK);
  if (!s) errstop(argv[1]);

  hash_fd=open(s->h32name,O_RDONLY);
  if (hash_fd==-1) errstop(s->h32name);

  if (read(hash_fd,hash_key,16)!=16) errstop(s->h32name);

  for (i=1;i<s->store_size;++i)
    switch(read_and_hash(s->fd,buf)) {
    default:
      fprintf(stderr,"%s: Incomplete read\n",argv[1]);
      return EXIT_FAILURE;
    case -1:
      errstop(s->name);
    case 512:
      if(read(hash_fd,buf+4,4)!=4) errstop(s->h32name);
      if (memcmp(buf,buf+4,4))
	fprintf(stderr,"Invalid hash in sector %lld\n",i);
    }

  close_store(s);
  return 0;
}
